C
C 
C ABSTRACT: 
C        DiffTop (vers. 1.0.0) program computes total cross section or 
C        differential distributions in pT and y for hadronic 
C        ttbar reactions at LO, approx NLO, approx NNLO, 
C        by using techniques of QCD threshold resummation
C        at LL, NLL and NNLL accurracy, in pico barn. 
C
C AUTHOR(S): 
C           Sven-Olaf Moch and Marco Guzzi, DESY HAMBURG
C 
C
C
C PAPER(S): 
C            'Top-quark pair production at hadron colliders: 
C             differential cross section and phenomenological applications with DiffTop',
C             by Marco Guzzi, Katerina Lipka and Sven-Olaf Moch,
C             e-Print: arXiv:1406.0386 [hep-ph].


      program difftop
      implicit none

      integer ptychoice, naccur 
      integer nqqborn, nggborn, nggnlo, nqqnlo, nggnnlo, nqqnnlo
      integer iloop, jloop
      integer kount, max
      integer ipdf, iset         
      integer PDFflag             
      integer ptsi, itsi, dimi, ptse, itse, dime, nel
      integer ptsb, itsb, dimb
      integer nalfas, Oalphas    
      integer sclset             ! Flag for the scale choice (0) scale = m; (1) scale = sqrt(m^2+pt^2)
      integer Npt, Nyt           ! Num. points in Pt and rapidity
      integer Nptp1, Nytp1       

      real*8 mq,muR,kR,muF       ! Ren./Fact. scales
      real*8 m2, scms, dscms, scmsroot, scale, alphas
      real*8 nlf
      real*8 shad, shadinel , shadel, betacap
      real*8 shaderr, shdinrr , shdelrr
      real*8 pi, ca, cf, co, ck, cqed, tf, nf 
      real*8 mfac,pt2,yyrap,pt2rt
      real*8 pt2rtmin, pt2rtmax      ! Min and Max values of pT
      real*8 yyrapmin, yyrapmax      ! MIn and Max valueas of y
      real*8 epsi, epse, epsb, delta
      real*8 s1,s2,s3,s4
      real*8 qcdl1, qcdl2, qcdl3     ! Flag for using pdf database


      character*10 q

      parameter (pi = 3.14159265359d0)

      common/result/s1,s2,s3,s4
      common/group/ca, cf, co, ck, cqed, tf, nf 
      common/m2only/m2
      common/sonly/scms
      common/include/ptychoice, naccur, nqqborn, nggborn,  
     #     nggnlo, nqqnlo, nggnnlo, nqqnnlo

      common/coupling/scale, alphas, nlf
      common/sclchoice/mq, muR, kR, muF, sclset     
      common/quark/q
      common/pdfdb/qcdl1,qcdl2,qcdl3,nalfas
      common/plus/delta
      common/channel/nel
      common/ptyblock/pt2,yyrap
      common/PDFname/PDFflag                        
      common/pdfiset/iset                           
      common/pdf/ipdf                               

c The function ff gives the total cross section 
c to be integrated by VEGAS

      external ff



      open (unit = 11, file ='difftop.in', status = 'old')

      open (unit = 12, file ='difftop.out')

      rewind 11
      rewind 12

      read(11,*) ptychoice      ! it decides which distrbution or Total Xsec

      read(11,*) PDFflag        ! it decides whether or not initializing LHAPDFs
      read(11,*) iset           ! it specifies the PDF set xxx.LHgrid from the input file 
      read(11,*) nalfas
      read(11,*) qcdl1          ! LambdaQCD 1-loop
      read(11,*) qcdl2          ! LambdaQCD 2-loop
      read(11,*) qcdl3          ! LambdaQCD 3-loop
      
      read(11,*) ipdf           ! identifies standalone PDFs when PDFflag=0 or LHAPDF when PDFflag=1
      read(11,*) naccur
      
      read(11,*) nggborn
      read(11,*) nqqborn
      read(11,*) nggnlo
      read(11,*) nqqnlo
      read(11,*) nggnnlo
      read(11,*) nqqnnlo
      read(11,*) sclset           ! scale choice (0) scale = m; (1) scale = sqrt(m^2+pt^2)
      
      read(11,*) mfac             ! it varies the factorization scale
      read(11,*) kR               ! it varies the renormalization scale
      read(11,*) mq
      read(11,*) scmsroot
      read(11,*) nlf
      read(11,*) q

      read(11,*) pt2rtmin        ! Min val in pT grid
      read(11,*) pt2rtmax        ! Max val in pT grid
      read(11,*) Npt             ! Num points for (Npt+1) pT-grid

      read(11,*) yyrapmin        ! Min val in y grid
      read(11,*) yyrapmax        ! Max val in y grid
      read(11,*) Nyt             ! Num points for (Ny+1) y-grid
 
      read(11,*) delta           ! cut-off parameter 
       
      
 87   format(a90)
     
      print*,'                                         '
      print*,'                                         '
      print*,'                                         '
      print*,'                                         '


      print*,'***********************************************'
      print*,'***********************************************'

      print*,'***********************************************'
      print*,'*      WELCOME to DIFFTOP version 1.0.1       *'
      print*,'*                                             *'
      print*,'* Authors: Marco Guzzi and Sven-Olaf Moch     *'
      print*,'*                                             *'
      print*,'*                                             *'
      print*,'* This code calculates the pT and y distrib.  *' 
      print*,'* in the 1PI kinematics and total incl. Xsec  *'
      print*,'* for heavy-flavor production at              *'
      print*,'* hadron colliders.                           *'
      print*,'*                                             *'
      print*,'* Please refer to arXiv[hep-ph] 1406.0386 by: *'
      print*,'* M. Guzzi, K. Lipka and S.-O. Moch           *'
      print*,'***********************************************'

      print*,'***********************************************'
      print*,'***********************************************'





C     CONTROL STATEMENTS
      if(PDFflag.eq.0.and.nalfas.eq.99.or.
     ~     PDFflag.eq.1.and.nalfas.ne.99) then
         print*,'choice of PDFflag = ', PDFflag
         print*,'choice of nalfas = ', nalfas
         print*,'ERROR: Incompatible choice of nalfas and PDFflag!!!'
         STOP
      endif

      if(PDFflag.eq.0.and.ipdf.eq.0.or.
     ~     PDFflag.eq.1.and.(ipdf.eq.1)) then
         print*,'choice of PDFflag = ', PDFflag
         print*,'choice of ipdf= ', ipdf
         print*,'ERROR: Incompatible choice of ipdf and PDFflag!!!'
         STOP
      endif

      print*,'                                         '
      print*,'                                         '
      print*,'                                         '
      print*,'                                         '


      if(PDFflag.eq.1) then
         print*,'Initializing LHA PDFs...'
         call InitialPDF
         call GetOrderAs(Oalphas)
         print*,'=============================='
         print*,'Alphas order from LHAPDF grid:', Oalphas
         print*,'=============================='
      endif
      


      print*,'*****************************************'
      print*,'*                                       *'
      print*,'*   PARAMETERS READING difftop.in file  *'
      print*,'*                                       *'
      print*,'*****************************************'


      print*,' choice of distrib. /pt/y/tot-pT/tot-y/ -> /0/1/2/3/:',
     >  ptychoice

      print*,' PDFs initialization: PDFflag = ',PDFflag
      print*,' parton PDF Set: .LHgrid number = ',iset
      print*,' nalfas flag for alphas evolution = ',nalfas !LHAPDF evol. nalfas=(99); internal alphas evol. nalfas=(2,3)
      print*,' ipdf flag for parton evolution = ',ipdf
      print*,' accurracy:  LL (0), NLL (1), NNLL (2): ', naccur
      print*,' include gg born (1) not (0): nggborn = ',nggborn
      print*,' include qq born (1) not (0): nqqborn = ',nqqborn
      print*,' include one loop gg (1) not (0): nggnlo = ',nggnlo
      print*,' include one loop qq (1) not (0): nqqnlo = ',nqqnlo
      print*,' include two loop gg (1) not (0): nggnnlo = ',nggnnlo
      print*,' include two loop qq (1) not (0): nqqnnlo = ',nqqnnlo
      print*,' scale choice: (0) m; (1) sqrt(m^2+pt^2) = ',sclset
      print*,' Factorization muF/scale = kF = ',mfac
      print*,' Renormalization muR/scale = kR = ',kR
      print*,' Mass of the produced heavy quark: mQ = ',mq
      print*,' center of mass energy: sqrt(S) = ', scmsroot
      print*,' the number of light flavors: nlf = ', nlf
      print*,' the heavy-quark produced is ',q
      print*,' Npt, pTmin, pTmax',Npt,pt2rtmin,pt2rtmax
      print*,' Nyt, ymin, ymax',Nyt,yyrapmin,yyrapmax
      print*,' the cutoff parameter delta =',delta


      print*,'======================================='
      print*,'======================================='
      print*,'======================================='

      

      if(mfac .ne.kR ) then
        print*,'INDEPENDENT VARIATIONS OF SCALES' 
        print*,'CANNOT BE COMPUTED IN THIS VERSION.'
        print*,'WORK IS ON GOING FOR THE NEXT DIFFTOP RELEASE.'
        print*,'THIS CALCULATION ENDS HERE!'
        STOP
      endif
      
c group structure constants ca = N of SU(N) = 3
      CA = 3.d0
      CF = 4.d0/3.d0
      CO = 24.d0
      CK = 8.d0/3.d0
      CQED = 80.d0/9.d0
      TF = 0.5d0
c put nf to number of light flavours
      nf = nlf

      m2 = mq*mq
      scms = scmsroot**2
      if (scms .le. 4.d0*m2) then 
         print*,'variables out of range'
         goto 39
      endif

c VEGAS parameters
      ptsi = 3000000
      itsi = 8
      epsi = 1d-20
      dimi = 4

      ptse = 750000
      itse = 8
      epse = 1d-20
      dime = 3

      ptsb = 100000
      itsb = 10
      epsb = 1d-20
      dimb = 3

c implement the various loops

      if (ptychoice.eq.0) then
c (i) loop over pt
c Set the pt grid 

        Nptp1=Npt+1

        do 21 iloop = 1,Nptp1
          shadinel = 0.d0
          shadel   = 0.d0
          shdinrr = 0.d0
          shdelrr= 0.d0
          pt2   = 0d0 
          yyrap = 0d0
c pT grid generation
          pt2rt=pt2rtmin + dble(iloop-1)*(pt2rtmax-pt2rtmin)/dble(Npt) 
          pt2 = pt2rt**2

c     calculate the mass factorization scale

            if (sclset.eq.0) then
               scale = dsqrt(m2)
            else if(sclset.eq.1) then
               scale = dsqrt(m2+pt2)
            else
               print*,'ERROR: Wrong assignment of sclset!'
               STOP
            endif

C           We set the factorization scale           
            muF=mfac*scale
            if (muF.le.1d0/4d0*mq.or.muF.ge.4d0*mq) then
               print*,'ERROR: Extreme choice of muF variation!'
               STOP 
            endif
            
         nel = 0
c born integration 

         if (nggborn .eq. 1 .or. nqqborn .eq. 1) then 
            call vegas(ff,epsb,dimb,ptsb,itsb,1,0)
            shadel = shadel + s1
	    shdelrr = shdelrr + s2
         endif

         nel = 1
c inelastic integration 
         if (nggnlo .eq. 1 .or. nqqnlo .eq. 1 .or. 
     ~        nggnnlo .eq. 1 .or. nqqnnlo .eq. 1) then 
            call vegas(ff,epsi,dimi,ptsi,itsi,1,0)
            shadinel = shadinel + s1
	    shdinrr = shdinrr + s2
         endif

         nel = 2
c elastic integration
         if (nggnlo .eq. 1 .or. nqqnlo .eq. 1 .or. 
     ~        nggnnlo .eq. 1 .or. nqqnnlo .eq. 1) then 
            call vegas(ff,epse,dime,ptse,itse,1,0)
            shadel =  shadel + s1
	    shdelrr = shdelrr + s2
         endif

         shad = shadinel + shadel
c linear
c         shaderr = shdinrr + shdelrr
c quadrature
         shaderr = sqrt(shdinrr**2 + shdelrr**2)

         write(12,100) pt2rt, shad, shaderr
         print*,'pT[GeV], dsigma/pT[pb/GeV], Err', pt2rt, shad, shaderr
 21   continue

      elseif(ptychoice.eq.1) then
c (ii) loop over y
c Set y range
         Nytp1=Nyt+1
         betacap = dsqrt(1d0-4d0*m2/scms)
         do 31 iloop = 1,Nytp1
            shadinel = 0.d0
            shadel   = 0.d0
            shdinrr = 0.d0
            shdelrr= 0.d0
            pt2   = 0d0 
            yyrap = 0.0d0
            yyrap=yyrapmin + dble(iloop-1)*(yyrapmax-yyrapmin)/dble(Nyt)



c calculate the mass factorization scale

            if (sclset.eq.0) then
               scale = dsqrt(m2)
            else if(sclset.eq.1) then
               scale = dsqrt(m2+pt2)
            else
               print*,'ERROR: Wrong assignment of sclset!'
               STOP
            endif

C           We set the factorization scale           
            muF=mfac*scale

            if (muF.le.1d0/4d0*mq.or.muF.ge.4d0*mq) then
               print*,'ERROR: Extreme choice of muF variation!'
               STOP 
            endif
            
         nel = 0
c born integration 
         if (nggborn .eq. 1 .or. nqqborn .eq. 1) then 
            call vegas(ff,epsb,dimb,ptsb,itsb,1,0)
            shadel = shadel + s1
	    shdelrr = shdelrr + s2
         endif

         nel = 1
c inelastic integration 
         if (nggnlo .eq. 1 .or. nqqnlo .eq. 1 .or. 
     ~        nggnnlo .eq. 1 .or. nqqnnlo .eq. 1) then 
            call vegas(ff,epsi,dimi,ptsi,itsi,1,0)
            shadinel = shadinel + s1
	    shdinrr = shdinrr + s2
         endif

         nel = 2
c elastic integration
         if (nggnlo .eq. 1 .or. nqqnlo .eq. 1 .or. 
     ~        nggnnlo .eq. 1 .or. nqqnnlo .eq. 1) then 
            call vegas(ff,epse,dime,ptse,itse,1,0)
            shadel =  shadel + s1
	    shdelrr = shdelrr + s2
         endif

         shad = shadinel + shadel
c linear
c         shaderr = shdinrr + shdelrr
c quadrature
         shaderr = sqrt(shdinrr**2 + shdelrr**2)

         write(12,100) yyrap, shad, shaderr
         print*,'y, dsigma/dy[pb], Err', yyrap, shad, shaderr
 31   continue

      else

         shadinel = 0.d0
         shadel   = 0.d0
         shdinrr = 0.d0
         shdelrr= 0.d0
         pt2   = 0d0 
         yyrap = 0d0
c calculate the mass factorization scale


         if (sclset.eq.0) then
            scale = dsqrt(m2)
         else if(sclset.eq.1) then
            scale = dsqrt(m2+pt2)
         else
            print*,'ERROR: Wrong assignment of sclset!'
            STOP
         endif
         
C     We set the factorization scale           
         muF=mfac*scale
         if (muF.le.1d0/4d0*mq.or.muF.ge.4d0*mq) then
            print*,'ERROR: Extreme choice of muF variation!'
            STOP 
         endif
         
         nel = 0
c born integration 
         if (nggborn .eq. 1 .or. nqqborn .eq. 1) then 
            call vegas(ff,epsb,dimb,ptsb,itsb,1,0)
            shadel = shadel + s1
	    shdelrr = shdelrr + s2
         endif

         nel = 2
c elastic integration
         if (nggnlo .eq. 1 .or. nqqnlo .eq. 1 .or. 
     ~        nggnnlo .eq. 1 .or. nqqnnlo .eq. 1) then 
            call vegas(ff,epse,dime,ptse,itse,1,0)
            shadel =  shadel + s1
	    shdelrr = shdelrr + s2
         endif

         nel = 1
c inelastic integration 
         if (nggnlo .eq. 1 .or. nqqnlo .eq. 1 .or. 
     ~        nggnnlo .eq. 1 .or. nqqnnlo .eq. 1) then 
            call vegas(ff,epsi,dimi,ptsi,itsi,1,0)
            shadinel = shadinel + s1
	    shdinrr = shdinrr + s2
         endif
	 
         shad = shadinel + shadel
c linear
c         shaderr = shdinrr + shdelrr
c quadrature
         shaderr = sqrt(shdinrr**2 + shdelrr**2)

         write(12,100) dsqrt(scms), shad, shaderr
         print*,'sqrt(S)[GeV], sigmatot [pb], Err', dsqrt(scms), shad
     >     ,shaderr
      endif

 39   continue
 100  format (4x,4(e12.5,4x))
 999  stop
      end
